@@ -95,6 +95,33 @@ sub cmd_inject {
     $self->install("--reinstall", @args);
 }
 
+sub cmd_pin {
+    my($self, @args) = @_;
+
+    unless (@args) {
+        die "Usage: carmel pin Module\@version ...\n";
+    }
+
+    my $snapshot = $self->snapshot
+      or die "Can't run carmel pin without snapshot. Run `carmel install` first.\n";
+
+    my $requirements = $self->requirements;
+    for my $arg (@args) {
+        my($module, $version) = split '@', $arg, 2;
+        unless (defined $version) {
+            die "Usage: carmel pin Module\@version ...\n";
+        }
+        my $dist = $snapshot->find($module)
+          or die "$module is not found in the snapshot.\n";
+        $requirements->add_string_requirement($module, "== $version");
+    }
+
+    # install with $requirements and $snapshot
+    my @artifacts = $self->install_from_cpanfile($requirements, $snapshot);
+    $self->dump_bootstrap(\@artifacts);
+    $self->save_snapshot(\@artifacts);
+}
+
 sub cmd_update {
     my($self, @args) = @_;
 
@@ -123,7 +150,7 @@ sub cmd_update {
     }
 
     # rebuild the snapshot
-    my @artifacts = $self->install_from_cpanfile($snapshot);
+    my @artifacts = $self->install_from_cpanfile($self->requirements, $snapshot);
     $self->dump_bootstrap(\@artifacts);
     $self->save_snapshot(\@artifacts);
 }
@@ -143,13 +170,13 @@ sub cmd_install {
 
     die "Usage: carmel install\n" if @args;
 
-    my @artifacts = $self->install_from_cpanfile($self->snapshot);
+    my @artifacts = $self->install_from_cpanfile($self->requirements, $self->snapshot);
     $self->dump_bootstrap(\@artifacts);
     $self->save_snapshot(\@artifacts);
 }
 
 sub install_from_cpanfile {
-    my($self, $snapshot) = @_;
+    my($self, $root_reqs, $snapshot) = @_;
 
     my $requirements = CPAN::Meta::Requirements->new;
     $self->resolve(
@@ -161,7 +188,7 @@ sub install_from_cpanfile {
             my($module, $want_version, $dist) = @_;
             $requirements->add_string_requirement($module => $want_version);
         },
-        1, # strict
+        $root_reqs,
         $snapshot,
     );
 
@@ -176,7 +203,7 @@ sub install_from_cpanfile {
     }
 
     my @artifacts;
-    $self->resolve(sub { push @artifacts, $_[0] }, undef, 0, $snapshot);
+    $self->resolve(sub { push @artifacts, $_[0] }, undef, $root_reqs, $snapshot);
 
     # $self->requirements has been upgraded at this point with the whole subreqs
     printf "---> Complete! %d cpanfile dependencies. %d modules installed.\n" .
@@ -595,15 +622,15 @@ sub merge_requirements {
         try {
             $reqs->add_string_requirement($module, $new);
         } catch {
-            my($err) = /illegal requirements: (.*) at/;
+            my($err) = /illegal requirements(?: .*?): (.*) at/;
             my $old = $reqs->requirements_for_module($module);
-            die "Found conflicting requirement for $module: '$old' <=> '$new' ($where): $err";
+            die "Found conflicting requirement for $module: '$old' <=> '$new' ($where): $err\n";
         };
     }
 }
 
 sub resolve_recursive {
-    my($self, $root_reqs, $requirements, $snapshot, $seen, $cb, $missing_cb, $strict, $depth) = @_;
+    my($self, $root_reqs, $requirements, $snapshot, $seen, $cb, $missing_cb, $depth) = @_;
 
     # TODO rather than mutating $root_reqs directly, we should create a new object
     # that allows accessing the result $requirements
@@ -616,11 +643,10 @@ sub resolve_recursive {
         my $dist;
         if ($dist = $self->find_in_snapshot($snapshot, $module, $root_reqs)) {
             $artifact = $self->repo->find_match($module, sub { $_[0]->distname eq $dist->name });
-            $artifact ||= $self->repo->find($module, $want_version) unless $strict;
         } elsif ($self->is_core($module, $want_version)) {
             next;
         } else {
-            $artifact = $self->repo->find($module, $want_version);
+            $artifact = $self->repo->find_match($module, sub { $self->accepts_all($root_reqs, $_[0]) });
         }
 
         # FIXME there's a chance different version of the same module can be loaded here
@@ -632,7 +658,7 @@ sub resolve_recursive {
             my $reqs = $artifact->requirements;
             $self->merge_requirements($root_reqs, $reqs, $artifact->distname);
 
-            $self->resolve_recursive($root_reqs, $reqs, $snapshot, $seen, $cb, $missing_cb, $strict, $depth + 1);
+            $self->resolve_recursive($root_reqs, $reqs, $snapshot, $seen, $cb, $missing_cb, $depth + 1);
         } else {
             $missing_cb->($module, $want_version, $dist, $depth);
         }
@@ -640,7 +666,7 @@ sub resolve_recursive {
 }
 
 sub resolve {
-    my($self, $cb, $missing_cb, $strict, $snapshot) = @_;
+    my($self, $cb, $missing_cb, $requirements, $snapshot) = @_;
     $missing_cb ||= sub {
         my($module, $want_version, $dist, $depth) = @_;
         die "Can't find an artifact for $module => $want_version\n" .
@@ -648,9 +674,10 @@ sub resolve {
     };
 
     $snapshot ||= $self->snapshot;
+    $requirements ||= $self->requirements;
 
-    $self->resolve_recursive($self->requirements, $self->requirements->clone, $snapshot,
-                             {}, $cb, $missing_cb, $strict, 0);
+    $self->resolve_recursive($requirements, $requirements->clone, $snapshot,
+                             {}, $cb, $missing_cb, 0);
 }
 
 sub find_in_snapshot {
